#! /usr/bin/python

"""//Web service to know the time elapsed since submission of a batch//
 
- File: get_elapsed_time.py

- Arguments - Using the HTTP GET method:
 - batch_id --- The batch identifier, as a string
 - current_time (Optional, for testing) ---  Retrieves batch's elapsed time since submission at specified fake "current_time", as an integer denoting the number of seconds since 1970-01-01 00:00:00 UTC. 
 - use_last_db_entry (Optional, for testing) --- If set to 1, use the timestamp found in the last database entry related to this batch as a fake current time.
 - > If the batch_id argument is missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">N</div>`` with N the time elapsed in seconds.
 
- Notes: 
 - A database configuration file named "info_db.cfg" must be present in the directory to connect to the SpeQuloS Info module database.

"""
import datetime
import time

import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)



def get_elapsed_time(conn,batch_id,current_time=int(time.time()) \
                        ,use_last_db_entry=False):
    """Function implementing the get_elapsed_time operation

    Arguments:
    conn -- The Info's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
    current_time (optional) -- A fake "current_time", as an integer denoting the number of seconds since 1970-01-01 00:00:00 UTC.
    use_last_db_time (optional) -- Boolean flag to use the timestamp found in the last databese entry related to this batch as a fake current time.
     
    The function returns the batch's elapsed time since its submission, as an integer 

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None    
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None
    if not isinstance(current_time,int) or current_time<=0:
        log.error("The current_time argument must be a positive integer !")
        return None
    if not isinstance(use_last_db_entry,bool):
        log.error("The use_last_db_entry argument must be a boolean !")
        return None
                        
    c=conn.cursor()
    log.info("Computing elapsed time since submission of batch %s (%s)" \
            % (batch_id,"current time: "+str(current_time) if not use_last_db_entry else "stopping at the last db entry"))
    
    try:
        c.execute("SELECT MIN(ts) FROM BatchCompletionHistory WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None            
   
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such batch identifier in the database : %s)" % batch_id )
        return None
    elif len(res)>1:
        log.error("Several results with same timestamp and batch_id %s in the database ! '(ts,batch_id)' should be BatchCompletionHistory PRIMARY KEY ! "% batch_id)
        return None
    
    try:
        first_time_seen=int(res[0][0])
        if first_time_seen<0:
            raise ValueError("Negative ts for first_time_seen !")
    except (ValueError,TypeError) as e:
        log.error("The 'ts' in BatchCompletionHistory table must be a positive or zero integer !")
        log.error(e)
        return None
    
    log.info("First time batch has been seen: %d" % (first_time_seen))

    if not use_last_db_entry:
        etime = current_time-first_time_seen 
    else:
        try:
            c.execute("SELECT MAX(ts) FROM BatchCompletionHistory WHERE batch_id='%s'" % batch_id)
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None       
        res = c.fetchall()
    
        if len(res)==0:
            log.error("No such batch identifier in the database : %s)" % batch_id)
            return None
        elif len(res)>1:
            log.error("Several results with same timestamp and batch_id %s in the database ! "% batch_id)
            return None

        try:
            last_time_seen=int(res[0][0])
            if last_time_seen<0:
                raise ValueError("Negative ts for last_time_seen !")
        except ValueError as e:
            log.error("The 'ts' in BatchCompletionHistory table must be a positive or zero integer !")
            log.error(e)
            return None          
                
        log.info("Last time seen: %d" % last_time_seen )        
        etime = last_time_seen-first_time_seen 

    if etime<0:
        log.error("We have a negative elapsed time !!")
        return None
    else:
        return etime     

if __name__=="__main__":
    cgi_func.print_header()

    if cgi_func.get_form_value('batch_id') is None:
        cgi_func.print_form(['batch_id','current_time','use_last_db_entry'])

    else:
        batch_id = cgi_func.get_form_value('batch_id')
            
        if cgi_func.get_form_value('use_last_db_entry') is not None:
            try:
                use_last_db=bool(cgi_func.get_form_value('use_last_db_entry'))
            except ValueError as e:
                log.error("use_last_db_entry argument must be 0 or 1")
                cgi_func.print_form(['batch_id','current_time','use_last_db_entry'])
            else:
                conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
                cgi_func.print_result( get_elapsed_time(conn,batch_id,use_last_db_entry=use_last_db))
                db_func.disconnect(conn)
         
        elif cgi_func.get_form_value('current_time') is not None:
            try:
                cur_time=int(cgi_func.get_form_value('current_time'))
            except ValueError as e:
                log.error("cur_time argument must be an integer")
                cgi_func.print_form(['batch_id','current_time','use_last_db_entry'])
            else:
                conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
                cgi_func.print_result( get_elapsed_time(conn,batch_id,current_time=cur_time))
                db_func.disconnect(conn)

        else:
            conn=db_func.connect(db_func.read_db_config("info_db.cfg"))
            cgi_func.print_result( get_elapsed_time(conn,batch_id))
            db_func.disconnect(conn)
        
    cgi_func.print_footer()

